
from rest_framework import status as st
from termcolor import cprint

class DifferentType(Exception):
    def __init__(self, field):
        message = f'Expected attribute of type: "{field}"'
        super().__init__(message)


class BaseResponse:
    def __init__(self, field='', code='', message='', key='status', extra_params:dict={}, status=st.HTTP_200_OK):
        """
        This Python function initializes an object with specified parameters and
        executes a response.
        
        Args:
          field: 
          code: 
          message: 
          key: 
          extra_params: 
          status: 
        """
        
        if not message: message = ''
        self.message = message
        self.field = field
        self.code = code
        self.key = key
        self.status = status
        self.extra_params = extra_params
        if not isinstance(extra_params, dict): raise DifferentType('dict')
        self.execute_response()

    def generate_exception_data(self):
        message = self.message
        data = {self.key: message}
        if self.code: data['code'] = self.code
        if self.extra_params: data.update(self.extra_params)
        return data
    
    def execute_response(self):...

    def good_response(self): ...
    
    def raise_exception(self): ...


from dj_utilitybox.response import base
from rest_framework import exceptions as rex
from rest_framework import status as st


class DynamicAPIException(rex.APIException):
    def __init__(self, detail=None, status=None):
        super().__init__(detail=detail)
        if status is not None:
            self.status_code = status
            

class BaseRaiseResponse(base.BaseResponse):
    def execute_response(self): self.raise_exception()
    
    def raise_exception(self):
        data = self.generate_exception_data()
        raise DynamicAPIException(data, status=self.status)


class CustomRaise(BaseRaiseResponse):
    def __init__(self, message: str, field='', code='', extra_params={}, status=st.HTTP_400_BAD_REQUEST):
        super().__init__(field, code=code, message=message, status=status, extra_params=extra_params)
    

class QueryParamNotFound(BaseRaiseResponse):
    def __init__(self, field: str, message='', code='', extra_params={}, status=st.HTTP_400_BAD_REQUEST):
        if not message: message = f"Queryparam '{field}' is required"
        else: message = f"{field} {message}"
        super().__init__(field, code=code, message=message, status=status, extra_params=extra_params)
        
        
class KwargNotFound(BaseRaiseResponse):
    def __init__(self, field: str, message='', code='', extra_params={}, status=st.HTTP_400_BAD_REQUEST):
        if not message: message = f"Kwarg '{field}' is required"
        else: message = f"{field} {message}"
        super().__init__(field, code=code, message=message, status=status, extra_params=extra_params)
        
        
class ModelNotFound(BaseRaiseResponse):
    def __init__(self, field: str, message='', code='', extra_params={}, status=st.HTTP_400_BAD_REQUEST):
        if not message: message = f"{field} don't exist"
        else: message = f"{field} {message}"
        super().__init__(field, code=code, message=message, status=status, extra_params=extra_params)
        

class ModelNotFound(BaseRaiseResponse):
    def __init__(self, field: str, message='', code='', extra_params={}, status=st.HTTP_400_BAD_REQUEST):
        if not message: message = f"{field} don't exist"
        else: message = f"{field} {message}"
        super().__init__(field, code=code, message=message, status=status, extra_params=extra_params)

# Cuando no viene algo en el data POST



from pathlib import Path
from setuptools import setup, find_packages

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

VERSION = '0.0.2'
DESCRIPTION = '...'
PACKAGE_NAME = 'django_utilitybox'
AUTHOR = 'Jose Angel Colin Najera'
EMAIL = 'josecolin99@gmail.com'
GITHUB_URL = 'https://github.com/Josecolin99/...'

setup(
    name = PACKAGE_NAME,
    packages = find_packages(),
    entry_points={
        "console_scripts": [
            "djangoutilitycmd=dj_utilitybox.__main__:main"
        ]
    },
    version = VERSION,
    license='MIT',
    description = DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    author = AUTHOR,
    author_email = EMAIL,
    url = GITHUB_URL,
    keywords = [
    ],
    install_requires=[ 
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ],
)